package io.aalam.testapp;

import io.aalam.common.HttpRequest;
import io.aalam.common.HttpResponse;
import io.aalam.common.HttpException;
import io.aalam.common.StatusTuple;
import io.aalam.common.Redis;
import io.aalam.common.serializers.JsonSerializer;

import io.netty.util.CharsetUtil;
import java.util.Map;
import java.util.HashMap;
import org.json.JSONObject;
import io.netty.handler.codec.http.HttpResponseStatus;

class ItemInfoJsonSerializer extends JsonSerializer {
    Map<String, String> item;

    public ItemInfoJsonSerializer(Map<String, String> item) {
        this.item = item;
    }
    public byte[] toJson(HttpRequest request, HttpResponse response) {
        return ("{\"name\": \"" + item.get("name") + "\"," +
                "\"type\": \"" + item.get("type") + "\"}").getBytes();
    }
}

class CreateItemInput {
    String name;
    String type;

    public CreateItemInput(String n, String t) {
        this.name = n;
        this.type = t;
    }

    public String name() {
        return this.name;
    }

    public String type() {
        return this.type;
    }
}

public class TestApp {
    public static Object sendHomePage(HttpRequest request) {
        /* We dont want to send the index page on a static prefixed URL,
         * hence we set the static_file attribute in the request object
         * and let the framework handle the response.
         */

        /* The index file will be placed by the aalam packager only if
         * it is put in the src/main/resources folder.
         */

        request.setStaticInfo("index_page", // The name of the resource.
                              "/index.html" // Path of the file in the
                                            // resources folder
                              );

        /* We are not returning null data, as the framework will take care
         * of sending an empty 200 response.
         */
        return null;
    }

    public static Object createItem(HttpRequest request) throws HttpException {

        /* Check the input data. It can either be in JSON format
         * or xml format
         */

        if (!request.headers().contains("Content-Type")) {
            throw new HttpException(HttpResponseStatus.BAD_REQUEST,
                                    "No input data present");
        }

        String content_type = request.headers().get("Content-Type");
        if (!content_type.startsWith("application/json") &&
            !content_type.startsWith("application/xml")) {
            throw new HttpException(HttpResponseStatus.BAD_REQUEST,
                                    "Invalid input data format");
        }

        byte[] input = request.content().toString(CharsetUtil.UTF_8).getBytes();

        CreateItemInput item = null;
        if (content_type.startsWith("application/json")) {
            JSONObject obj = new JSONObject(request.content().toString(CharsetUtil.UTF_8));
            item = new CreateItemInput(
               obj.getString("name"), obj.getString("type"));
        } else {
            /*TODO: parse xml input*/
            item = null;
        }

        /* Get the email id of the user creating this item. */
        String userEmail = request.auth().email();

        /* We store this item in redis. */
        Map<String, String> data = new HashMap<String, String>();
        data.put("type", item.type());
        data.put("owner", userEmail);
        Redis.Client().hmset(item.name(), data);

        /* If we return a tuple with null data, just the status code will
         * be set in the response.
         */
        return new StatusTuple(HttpResponseStatus.CREATED, null);
    }


    public static Object getItem(HttpRequest request, String itemName) {
        /* Note the parameter, 'item_name'. The name of the parameter
         * can be anything. The parameters appear in the same order as
         * used in the url. The value is as it is in the actual path,
         * example: for /aalam/testapp/some_name,
         * 'itemName' will be 'some_name'.
         */

        /* We get the data back from redis and return a dictionary. This data
         * will be passed to get_item_serializer.
         */
        Map<String, String> itemInfo = Redis.Client().hgetall(itemName);

        /* This method returns the serializer object which serializes the
         * the map object into Json content
         */
        return new ItemInfoJsonSerializer(itemInfo);
    }

    public static Object updateItem(HttpRequest request, String itemName) throws HttpException {
        /* 'itemName' is set in the same way as it is for getItem().
         */

        /* we expect the new type in the request parameter. If not present
         * throw an error.
         */
        if (!request.params().containsKey("type"))
            throw new HttpException(HttpResponseStatus.BAD_REQUEST,
                                    "Invalid usage");

        /* Check if the owner of this item is updating this item. Else
         * throw an error.
         */
        String userEmail = request.auth().email();

        if (Redis.Client().hget(itemName, "owner") != userEmail)
            throw HttpException.forbidden("");

        /* We can update the item now, as the user is verified.
        Redis.Client().hset(itemName, "type", request.params.get("type"))

        /* This just sends a 200 OK response. */
        return null;
    }
}
